library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

Library UNISIM;
use UNISIM.vcomponents.all;

entity flash_test_v1_0 is
	generic (
		-- Parameters of Axi Slave Bus Interface S00_AXI
		C_S00_AXI_DATA_WIDTH	: integer	:= 32;
		C_S00_AXI_ADDR_WIDTH	: integer	:= 26
	);
	port (
		-- Users to add ports here
    CFGCLK      : out std_logic;
    EOS         : out std_logic;

    o_flash_dq3 : out std_logic; -- always HI
    o_flash_dq2 : out std_logic; -- always HI
    i_flash_dq1 : in std_logic; 
    o_flash_dq0 : out std_logic;
    o_flash_ss  : out std_logic;
		-- User ports ends
    --------------------------------------------------
		-- Do not modify the ports beyond this line
    --------------------------------------------------


		-- Ports of Axi Slave Bus Interface S00_AXI
		s00_axi_aclk	: in std_logic;
		s00_axi_aresetn	: in std_logic;
		s00_axi_awaddr	: in std_logic_vector(C_S00_AXI_ADDR_WIDTH-1 downto 0);
		s00_axi_awprot	: in std_logic_vector(2 downto 0);
		s00_axi_awvalid	: in std_logic;
		s00_axi_awready	: out std_logic;
		s00_axi_wdata	: in std_logic_vector(C_S00_AXI_DATA_WIDTH-1 downto 0);
		s00_axi_wstrb	: in std_logic_vector((C_S00_AXI_DATA_WIDTH/8)-1 downto 0);
		s00_axi_wvalid	: in std_logic;
		s00_axi_wready	: out std_logic;
		s00_axi_bresp	: out std_logic_vector(1 downto 0);
		s00_axi_bvalid	: out std_logic;
		s00_axi_bready	: in std_logic;
		s00_axi_araddr	: in std_logic_vector(C_S00_AXI_ADDR_WIDTH-1 downto 0);
		s00_axi_arprot	: in std_logic_vector(2 downto 0);
		s00_axi_arvalid	: in std_logic;
		s00_axi_arready	: out std_logic;
		s00_axi_rdata	: out std_logic_vector(C_S00_AXI_DATA_WIDTH-1 downto 0);
		s00_axi_rresp	: out std_logic_vector(1 downto 0);
		s00_axi_rvalid	: out std_logic;
		s00_axi_rready	: in std_logic;
		
		
		o_flash_dbg : out std_logic_vector(3 downto 0)
	);
end flash_test_v1_0;

architecture arch_imp of flash_test_v1_0 is

	-- component declaration
	component flash_test_v1_0_S00_AXI is
		generic (
		C_S_AXI_DATA_WIDTH	: integer	:= 32;
		C_S_AXI_ADDR_WIDTH	: integer	:= 26
		);
		port (
    o_usr_access : out std_logic;
    o_usr_erase : out std_logic;
    o_usr_we : out std_logic;
    i_write_ok  : in std_logic;
    i_read_ok   : in std_logic;
    i_flash_rdy : in std_logic;
    o_wr_addr : out std_logic_vector(24 downto 0);
    o_wr_data : out std_logic_vector(31 downto 0);
    o_rd_addr : out std_logic_vector(24 downto 0);
    i_rd_data : in  std_logic_vector(31 downto 0);

		S_AXI_ACLK	: in std_logic;
		S_AXI_ARESETN	: in std_logic;
		S_AXI_AWADDR	: in std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
		S_AXI_AWPROT	: in std_logic_vector(2 downto 0);
		S_AXI_AWVALID	: in std_logic;
		S_AXI_AWREADY	: out std_logic;
		S_AXI_WDATA	: in std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
		S_AXI_WSTRB	: in std_logic_vector((C_S_AXI_DATA_WIDTH/8)-1 downto 0);
		S_AXI_WVALID	: in std_logic;
		S_AXI_WREADY	: out std_logic;
		S_AXI_BRESP	: out std_logic_vector(1 downto 0);
		S_AXI_BVALID	: out std_logic;
		S_AXI_BREADY	: in std_logic;
		S_AXI_ARADDR	: in std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
		S_AXI_ARPROT	: in std_logic_vector(2 downto 0);
		S_AXI_ARVALID	: in std_logic;
		S_AXI_ARREADY	: out std_logic;
		S_AXI_RDATA	: out std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
		S_AXI_RRESP	: out std_logic_vector(1 downto 0);
		S_AXI_RVALID	: out std_logic;
		S_AXI_RREADY	: in std_logic
		);
	end component flash_test_v1_0_S00_AXI;


component flash_ctrl is
  port
  (
    rst             : in std_logic;
    clk             : in std_logic;
    
    usr_access_i    : in std_logic;
    usr_erase_i     : in std_logic;
    usr_we_i        : in std_logic;
    usr_wr_adr_i    : in std_logic_vector(24 downto 0);
    usr_dati_i      : in std_logic_vector(31 downto 0);
    usr_rd_adr_i    : in std_logic_vector(24 downto 0);
    usr_dato_o      : out std_logic_vector(31 downto 0);
    write_ok_o      : out std_logic;
    read_ok_o       : out std_logic;

    flash_rdy_o     : out std_logic;
    flash_s_n_o     : out std_logic;
    flash_c_o       : out std_logic;
    flash_hold_n_o  : out std_logic;
    flash_w_n_o     : out std_logic;
    flash_d_o       : out std_logic;
    flash_q_i       : in std_logic
  );
end component;


signal s_usr_access: std_logic;
signal s_usr_erase : std_logic;
signal s_usr_we    : std_logic;
signal s_write_ok  : std_logic;
signal s_read_ok   : std_logic;
signal s_flash_rdy : std_logic;
signal s_wr_addr   : std_logic_vector(24 downto 0);
signal s_wr_data   : std_logic_vector(31 downto 0);
signal s_rd_addr   : std_logic_vector(24 downto 0);
signal s_rd_data   : std_logic_vector(31 downto 0);
signal s_flash_clk : std_logic;

signal s_flash_dq3  : std_logic;
signal s_flash_dq2  : std_logic;
signal s_flash_dq1  : std_logic;
signal s_flash_dq0  : std_logic;
signal s_flash_ss   : std_logic;

signal S_s00_axi_awready : std_logic;
signal S_s00_axi_awvalid : std_logic;
signal S_s00_axi_arready : std_logic;
signal S_s00_axi_arvalid : std_logic;



begin

S_s00_axi_awvalid <= s00_axi_awvalid;
s00_axi_awready <= S_s00_axi_awready;

S_s00_axi_arvalid <= s00_axi_arvalid;
s00_axi_arready <= S_s00_axi_arready;

STARTUPE2_inst : STARTUPE2
generic map
(
  PROG_USR => "FALSE",  -- Activate program event security feature. Requires encrypted bitstreams.
  SIM_CCLK_FREQ => 0.0  -- Set the Configuration Clock Frequency(ns) for simulation.
)
port map
(
  CFGCLK => CFGCLK,         -- 1-bit output: Configuration main clock output
  CFGMCLK => open,          -- 1-bit output: Configuration internal oscillator clock output
  EOS => EOS,               -- 1-bit output: Active high output signal indicating the End Of Startup.
  PREQ => open,             -- 1-bit output: PROGRAM request to fabric output
  CLK => '0',               -- 1-bit input: User start-up clock input
  GSR => '0',               -- 1-bit input: Global Set/Reset input (GSR cannot be used for the port name)
  GTS => '0',               -- 1-bit input: Global 3-state input (GTS cannot be used for the port name)
  KEYCLEARB => '0',         -- 1-bit input: Clear AES Decrypter Key input from Battery-Backed RAM (BBRAM)
  PACK => '0',              -- 1-bit input: PROGRAM acknowledge input
  USRCCLKO => s_flash_clk,  -- 1-bit input: User CCLK input
  USRCCLKTS => '0',         -- 1-bit input: User CCLK 3-state enable input
  USRDONEO => '1',          -- 1-bit input: User DONE pin output control
  USRDONETS => '1'          -- 1-bit input: User DONE 3-state enable output
);


-- Instantiation of Axi Bus Interface S00_AXI
flash_test_v1_0_S00_AXI_inst:
component flash_test_v1_0_S00_AXI
generic map
(
  C_S_AXI_DATA_WIDTH	=> C_S00_AXI_DATA_WIDTH,
  C_S_AXI_ADDR_WIDTH	=> C_S00_AXI_ADDR_WIDTH
)
port map
(
  o_usr_access  => s_usr_access,
  o_usr_erase   => s_usr_erase ,
  o_usr_we      => s_usr_we    ,
  i_write_ok    => s_write_ok  ,
  i_read_ok     => s_read_ok   ,
  i_flash_rdy   => s_flash_rdy ,
  o_wr_addr     => s_wr_addr   ,
  o_wr_data     => s_wr_data   ,
  o_rd_addr     => s_rd_addr   ,
  i_rd_data     => s_rd_data   ,

  S_AXI_ACLK	=> s00_axi_aclk,
  S_AXI_ARESETN	=> s00_axi_aresetn,
  S_AXI_AWADDR	=> s00_axi_awaddr,
  S_AXI_AWPROT	=> s00_axi_awprot,
  S_AXI_AWVALID	=> S_s00_axi_awvalid,
  S_AXI_AWREADY	=> S_s00_axi_awready,
  S_AXI_WDATA	=> s00_axi_wdata,
  S_AXI_WSTRB	=> s00_axi_wstrb,
  S_AXI_WVALID	=> s00_axi_wvalid,
  S_AXI_WREADY	=> s00_axi_wready,
  S_AXI_BRESP	=> s00_axi_bresp,
  S_AXI_BVALID	=> s00_axi_bvalid,
  S_AXI_BREADY	=> s00_axi_bready,
  S_AXI_ARADDR	=> s00_axi_araddr,
  S_AXI_ARPROT	=> s00_axi_arprot,
  S_AXI_ARVALID	=> S_s00_axi_arvalid,
  S_AXI_ARREADY	=> S_s00_axi_arready,
  S_AXI_RDATA	=> s00_axi_rdata,
  S_AXI_RRESP	=> s00_axi_rresp,
  S_AXI_RVALID	=> s00_axi_rvalid,
  S_AXI_RREADY	=> s00_axi_rready
);

o_flash_dq3 <= s_flash_dq3;
o_flash_dq2 <= s_flash_dq2;
s_flash_dq1 <= i_flash_dq1;
o_flash_dq0 <= s_flash_dq0;
o_flash_ss  <= s_flash_ss;

o_flash_dbg <= S_s00_axi_awready & S_s00_axi_awvalid & S_s00_axi_arready & S_s00_axi_arvalid;

	-- Add user logic here
inst_flash:
component flash_ctrl port map
(
    rst             => s00_axi_aresetn,
    clk             => s00_axi_aclk,
    usr_access_i    => s_usr_access,
    usr_erase_i     => s_usr_erase,
    usr_we_i        => s_usr_we,
    usr_wr_adr_i    => s_wr_addr,
    usr_dati_i      => s_wr_data,
    usr_rd_adr_i    => s_rd_addr,
    usr_dato_o      => s_rd_data,
    write_ok_o      => s_write_ok,
    read_ok_o       => s_read_ok,

    flash_rdy_o     => s_flash_rdy,
    flash_c_o       => s_flash_clk,
    flash_s_n_o     => s_flash_ss,
    flash_hold_n_o  => s_flash_dq3,
    flash_w_n_o     => s_flash_dq2,
    flash_q_i       => s_flash_dq1,
    flash_d_o       => s_flash_dq0
);
	-- User logic ends

end arch_imp;