----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 11/02/2016 09:54:27 AM
-- Design Name: 
-- Module Name: TB_efm03_sim_wrapper - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity TB_efm03_sim_wrapper is
--  Port ( );
end TB_efm03_sim_wrapper;

architecture Behavioral of TB_efm03_sim_wrapper is

component efm03_sim_wrapper is
  port (
    i_fx3_flag : in STD_LOGIC_VECTOR ( 3 downto 0 );
    i_fx3_pclk : in STD_LOGIC;
    i_reset : in STD_LOGIC;
    io_fx3_fdata : inout STD_LOGIC_VECTOR ( 31 downto 0 );
    o_fx3_faddr : out STD_LOGIC_VECTOR ( 1 downto 0 );
    o_fx3_pclk : out STD_LOGIC;
    o_fx3_pktend_n : out STD_LOGIC;
    o_fx3_slcs_n : out STD_LOGIC;
    o_fx3_sloe_n : out STD_LOGIC;
    o_fx3_slrd_n : out STD_LOGIC;
    o_fx3_slwr_n : out STD_LOGIC;
    sys_clk : in STD_LOGIC
  );
end component;

signal s_clk_n        : STD_LOGIC := '1';
signal s_clk_p        : STD_LOGIC := '0';
signal s_reset        : STD_LOGIC := '0';
signal s_fx3_plclk    : STD_LOGIC := '0';
signal s_fx3_flag     : STD_LOGIC_VECTOR (3 downto 0) := "0000";
signal s_fx3_slcs_n   : STD_LOGIC := '1';
signal s_fx3_slrd_n   : STD_LOGIC := '1';
signal s_fx3_slwr_n   : STD_LOGIC := '1';
signal s_fx3_sloe_n   : STD_LOGIC := '1';
signal s_fx3_pktend_n : STD_LOGIC := '1';
signal s_fx3_faddr    : STD_LOGIC_VECTOR (1 downto 0);
signal s_fx3_fdata    : STD_LOGIC_VECTOR (31 downto 0);
signal s_udk3_data    : STD_LOGIC_VECTOR (31 downto 0);
signal s_dummy_data   : unsigned (31 downto 0) := (others => '0');
signal s_axi_usb_fill : unsigned (31 downto 0) := (others => '0');

constant c_clk_period : time := 10ns;

-- FX3 interface
begin

-- 100MHz clock
p_clk_gen:
process begin
  wait for c_clk_period/2;
  s_clk_p     <= not s_clk_p;
end process;
 s_clk_n <= not s_clk_p;


s_fx3_fdata <= s_udk3_data when s_fx3_sloe_n = '0' else (others => 'Z');

p_stimuli:
process begin
  s_fx3_flag  <= "0000";
  wait until (s_clk_p'event and s_clk_p = '1');
  wait until (s_clk_p'event and s_clk_p = '1');
  wait for 25*c_clk_period; -- 250ns
  s_reset <= '1';

  wait for 750ns;           -- 1 us
----------------------------------------------
-- Testing WRITE Commands
  s_fx3_flag  <= "0010";
  wait for 9us;             -- 10 us 
  s_fx3_flag  <= "0001";
  wait for 20us;            -- 30 us
  s_fx3_flag  <= "0010";
  wait for 30us;            -- 60 us
  s_fx3_flag  <= "0000";
  wait for 25us;            -- 85 us
----------------------------------------------
-- Testing READ Commands
  s_fx3_flag  <= "0010";
  wait for 5us;             -- 90 us
  s_fx3_flag  <= "0000";
  wait for 30us;            -- 120 us
  s_fx3_flag  <= "1000";
  wait for 10us;            -- 130 us
  s_fx3_flag  <= "0100";
  wait for 70us;            -- 200 us
  s_fx3_flag  <= "1000";
  wait for 50us;            -- 250 us
  s_fx3_flag  <= "0000";
  -- End of test
------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
  wait;
end process;

process begin
  s_udk3_data <= (others => '0');
  wait for 350ns;
------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
--------------------------------- WRITE COMMANDS -----------------------------------
------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
-- Command 1
-- Write 1024 bytes to BRAM
----------------------------------------------
  wait until (s_fx3_slrd_n = '0');
  for i in 0 to 2 loop
    wait until (s_clk_p'event and s_clk_p = '1');
  end loop;
  s_udk3_data <= x"C0_00_40_00";    
  wait for c_clk_period;
  s_udk3_data <= x"00_00_00_00";
  wait for c_clk_period;
  -- dummy header data
  for i in 0 to 2045 loop
    s_dummy_data <= s_dummy_data + 1;
    s_udk3_data <= STD_LOGIC_VECTOR(s_dummy_data + 1);
    wait for c_clk_period;
  end loop;

  s_dummy_data  <= (others => '0');

  for i in 0 to 1 loop
    wait until (s_fx3_slrd_n = '0');
    for i in 0 to 2 loop
      wait until (s_clk_p'event and s_clk_p = '1');
    end loop;
  -- Data from PC for the previous write cmd
    for i in 0 to 2047 loop
        s_dummy_data <= s_dummy_data + 1;
        s_udk3_data <= STD_LOGIC_VECTOR(s_dummy_data + 1);
        wait for c_clk_period;
    end loop;
  end loop;
  s_dummy_data  <= (others => '0');
--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------


--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
----------------------------------- READ COMMANDS ------------------------------------
--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
-- Command 2
-- Read 1024 bytes from BRAM
----------------------------------------------
  wait until (s_clk_p'event and s_clk_p = '1');
  wait until (s_fx3_slrd_n = '0');
  for i in 0 to 2 loop
    wait until (s_clk_p'event and s_clk_p = '1');
  end loop;
  s_udk3_data <= x"40_00_40_00";
  wait for c_clk_period;
  s_udk3_data <= x"00_00_00_00";
  wait for c_clk_period;
  -- dummy header data
  for i in 0 to 2045 loop
   s_dummy_data <= s_dummy_data + 1;
   s_udk3_data  <= STD_LOGIC_VECTOR(s_dummy_data + 1);
   wait for c_clk_period;
  end loop;
  s_dummy_data  <= (others => '0');
  s_udk3_data   <= (others => '0');
  wait for 2us;

  wait;
end process;

--------------------------------------------------------------------------------------                                                                                        
--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------

DUT:
component efm03_sim_wrapper port map
(
      sys_clk               => s_clk_p        ,
      i_reset               => s_reset        ,
      -- FX3 interface      => 
      i_fx3_pclk            => s_clk_n        ,
      o_fx3_pclk            => s_fx3_plclk    ,
      i_fx3_flag            => s_fx3_flag     ,
      o_fx3_slcs_n          => s_fx3_slcs_n   ,
      o_fx3_slrd_n          => s_fx3_slrd_n   ,
      o_fx3_slwr_n          => s_fx3_slwr_n   ,
      o_fx3_sloe_n          => s_fx3_sloe_n   ,
      o_fx3_pktend_n        => s_fx3_pktend_n ,
      o_fx3_faddr           => s_fx3_faddr    ,
      io_fx3_fdata          => s_fx3_fdata
);
end Behavioral;
